--Events for Classic Rome

-- this will print to the lua console
print "You should see this in lua console if this worked"

-- The require command allows the access of code written in another file
-- At the moment, the lua console will only look in the lua directory,
-- not in the directory where Events.lua is located
local civlua = require "civlua"
local func = require "functions"
-- The assignment tells how we access the required functions.
-- e.g. to use functionInCivlua(), we write civlua.functionInCivlua()


-- This implements the justOnce function.  Not part of lesson 2
local state = {}
local justOnce = function (key, f) civlua.justOnce(civlua.property(state, key), f)
end

local campaignCost = 5
local evenTurnLeader = "Scipio"
local oddTurnLeader = "Fabius"
local romans = civ.getTribe(1)
local celts = civ.getTribe(7)
local chariot =civ.getUnitType(16)
local milan = civ.getTile(38,12,0).city
local chariotSquare = civ.getTile(37,15,0)

local seleucids = civ.getTribe(5)
local seleucidRulers = {
[-281] = {name = "Antiochus I Soter", female = false},
[-278] = {name = "Antiochus I Soter", female = false}, -- this is the year the game starts
[-261] = {name = "Antiochus II Theos", female = false},
[-246] = {name = "Seleucus II Callinicus", female = false},
[-225] = {name = "Seleucus III Ceraunus", female = false},
-- this makes the point
[-126] = {name = "Cleopatra Thea", female = true},
-- have a female ruler to test, though
}--close seleucidRulers

local elephant = civ.getUnitType(17)
local catapult = civ.getUnitType(23)
local elephantCampaignCost = 5
local catapultCampaignCost = 4

local function doWhenUnitKilled(loser,winner)
    --[[loser.owner.money = math.max(loser.owner.money -campaignCost,0)
    winner.owner.money = math.max(winner.owner.money-campaignCost,0)]]
	for __,unit in pairs({loser,winner}) do
		if unit.type == elephant then
			unit.owner.money = math.max(unit.owner.money-elephantCampaignCost,0)
		elseif unit.type == catapult then
			unit.owner.money = math.max(unit.owner.money-catapultCampaignCost,0)
		else
			unit.owner.money = math.max(unit.owner.money - unit.type.move/totpp.movementMultipliers.aggregate,0)
		end
	end
end
civ.scen.onUnitKilled(doWhenUnitKilled)

local barbarians = civ.getTribe(0)
local archers = civ.getUnitType(4)
local horsemen = civ.getUnitType(15)

local barbTable = {
{location = {44,12}, unit = archers},
{location = {35,39}, unit = horsemen},
{location = {64,16}, unit = horsemen},
}-- close barbTable
local desertTerrain = 0

local function doThisOnTurn(turn)
	for __, barbCreate in pairs(barbTable) do
		local tile = civ.getTile(barbCreate.location[1],barbCreate.location[2],0)
		if tile.defender == barbarians or tile.defender == nil then
			civ.createUnit(barbCreate.unit,barbarians,tile)
		end
	end
	for unit in civ.iterateUnits() do
		if unit.location.terrainType % 16 == desertTerrain then
			local newDamage = math.floor(unit.hitpoints/2)
			unit.damage = unit.damage+newDamage
		end
	end
    if turn % 2 == 0 then
        --turn is even
        romans.leader.name = evenTurnLeader
    end
    if turn %2 == 1 then
        -- turn is odd
        romans.leader.name = oddTurnLeader
    end
	if (milan.owner == celts) and (chariotSquare.defender == nil or chariotSquare.defender == celts) then
        newChariot = civ.createUnit(chariot,celts,chariotSquare)
        newChariot.veteran = false
        newChariot.homeCity = nil
    end
	if seleucidRulers[civ.getGameYear()] then
		-- if there is no entry for this index, it returns nil, which the if statement counts as false.
		seleucids.leader.name = seleucidRulers[civ.getGameYear()].name
		seleucids.leader.female = seleucidRulers[civ.getGameYear()].female
	end --if seleucidRulers[civ.getGameYear()]
end--doThisOnTurn

local function doOnActivation(unit,source)
-- remove all barbarian bombers
local blockerUnit = civ.getUnitType(28)
for checkUnit in civ.iterateUnits() do
    if checkUnit.owner == civ.getTribe(0) and checkUnit.type ==blockerUnit then
        civ.deleteUnit(checkUnit)
    end
end

if unit.type.domain == 0 and unit.location.terrainType % 16 == 10 and not(unit.type.role ==5)  then
    for __, offsetPair in pairs({{2,0},{1,1},{0,2},{-1,1},{-2,0},{-1,-1},{0,-2},{1,-1},}) do
        local x = unit.location.x
        local y = unit.location.y
        local z = unit.location.z
        local tile = civ.getTile(x+offsetPair[1],y+offsetPair[2],z)
        if tile.terrainType % 16 ~= 10 then
            civ.createUnit(blockerUnit, civ.getTribe(0),tile)
        end
	end
end
end

civ.scen.onActivateUnit(doOnActivation)

civ.scen.onTurn(doThisOnTurn)


-- These are necessary to 'save' and 'load' the data associated with the justOnce function.
--  Not Part of lesson 2.
civ.scen.onLoad(function (buffer) state = civlua.unserialize(buffer) end)
 civ.scen.onSave(function () return civlua.serialize(state) end) 


